@@ -6,7 +6,7 @@ from PySide.QtGui import *
 from PySide.QtMaemo5 import QMaemo5ValueButton, QMaemo5ListPickSelector
 
 # Some global variables
-GnGeoGuiVersion = "0.0.6-0"
+GnGeoGuiVersion = "0.0.6-1"
 CommentChar = '#'
 OptionChar =  ' '
 Home = os.path.expanduser("~")
@@ -20,9 +20,11 @@ P2joybuttons = ""
 P1hotbuttons0 = ""
 P1hotbuttons1 = ""
 P1hotbuttons2 = ""
+P1hotbuttons3 = ""
 P2hotbuttons0 = ""
 P2hotbuttons1 = ""
 P2hotbuttons2 = ""
+P2hotbuttons3 = ""
 quitbutton = ""
 resetbutton = ""
 savestatebutton = ""
@@ -84,7 +86,8 @@ class Ui_MainWindow(QMainWindow):
 	AllGnGeoOpts = [ "68kclock", "autoframeskip", "biospath", "country", "effect", "fullscreen", "hwsurface", 
 	                 "interpolation", "joystick", "p1joy", "p1joydev", "p2key", "p2joy", "p2joydev", "p2key", "raster", 
 	                 "romrcdir", "samplerate", "scale", "showfps", "sleepidle", "sound", "system", "z80clock", "quit_key",
-	                 "reset_key", "load_state_key","save_state_key","p1hotkey0","p1hotkey1","p2hotkey0", "p2hotkey1"]
+	                 "reset_key", "load_state_key","save_state_key","p1hotkey0","p1hotkey1","p1hotkey2","p1hotkey3",
+			 "p2hotkey0", "p2hotkey1","p2hotkey2","p2hotkey3" ]
 	for GnGeoOpt in AllGnGeoOpts:
 		try:
 			 self.options[GnGeoOpt]
@@ -585,6 +588,20 @@ class ConfigWindow(QMainWindow):
 				tot_sleutel = tot_sleutel + sleutel + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p1hotkey1"] = tot_sleutel
+	if P1hotbuttons2 != "":
+		tot_sleutel=""
+		for sleutel in P1hotbuttons2:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p1hotkey2"] = tot_sleutel
+	if P1hotbuttons3 != "":
+		tot_sleutel=""
+		for sleutel in P1hotbuttons3:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p1hotkey3"] = tot_sleutel
 
 	if P2hotbuttons0 != "":
 		tot_sleutel=""
@@ -600,6 +617,20 @@ class ConfigWindow(QMainWindow):
 				tot_sleutel = tot_sleutel + sleutel + ","
 		tot_sleutel = tot_sleutel[:-1]
 		self.options["p2hotkey1"] = tot_sleutel
+	if P2hotbuttons2 != "":
+		tot_sleutel=""
+		for sleutel in P2hotbuttons2:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p2hotkey2"] = tot_sleutel
+	if P2hotbuttons3 != "":
+		tot_sleutel=""
+		for sleutel in P2hotbuttons3:
+			if sleutel != "":
+				tot_sleutel = tot_sleutel + sleutel + ","
+		tot_sleutel = tot_sleutel[:-1]
+		self.options["p2hotkey3"] = tot_sleutel
 
 	if (ExecPath != "") or ((doExit == "true") and (self.gui_opts["exit_after_launch"] == "false")) or ((doExit == "false") and (self.gui_opts["exit_after_launch"] == "true")):
 		# Change in gngeo-gui config, we need to save it
@@ -709,6 +740,12 @@ class keyWindow(QDialog):
 	if len(self.P1Keys) < 12:
 		self.P1Keys.append(-1)
 	P1hotkey1=self.P1Keys[11]
+	if len(self.P1Keys) < 13:
+		self.P1Keys.append(-1)
+	P1hotkey2=self.P1Keys[12]
+	if len(self.P1Keys) < 14:
+		self.P1Keys.append(-1)
+	P1hotkey3=self.P1Keys[13]
 	# P2 keys
 	P2Akey=self.P2Keys[0]
 	P2Bkey=self.P2Keys[1]
@@ -726,13 +763,19 @@ class keyWindow(QDialog):
 	if len(self.P2Keys) < 12:
 		self.P2Keys.append(-1)
 	P2hotkey1=self.P2Keys[11]
+	if len(self.P2Keys) < 13:
+		self.P2Keys.append(-1)
+	P2hotkey2=self.P2Keys[12]
+	if len(self.P2Keys) < 14:
+		self.P2Keys.append(-1)
+	P2hotkey3=self.P2Keys[13]
 	# Layout
 	area = QScrollArea(self)
 	lay = QVBoxLayout(self)
 	lay.addWidget(area)
 	wg = QWidget()
 	area.setWidget(wg)
-	wg.resize(750,1100);
+	wg.resize(750,1500);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -747,7 +790,7 @@ class keyWindow(QDialog):
 	grid.addWidget(QLabel("<center>[pick]</center>"), 1, 4)
 	# Add button list
         cntr=2
-	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1','RESET','EXIT','SAVEST.','LOADST.']:
+	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1','HOTKEY2','HOTKEY3','RESET','EXIT','SAVEST.','LOADST.']:
 		grid.addWidget(QLabel(pButton), cntr, 0)
 		cntr=cntr+1
 	# A-button P1
@@ -926,6 +969,22 @@ class keyWindow(QDialog):
 	grid.addWidget(self.P1Hotkey1SpinBox, 13, 1)
 	self.P1Hotkey1Button = MyKeyPressed("key",self)
 	grid.addWidget(self.P1Hotkey1Button, 13, 2)
+	# Hot-key2 P1
+	self.P1Hotkey2SpinBox = QDoubleSpinBox()
+	self.P1Hotkey2SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotkey2SpinBox.setValue(float(P1hotkey2))
+	self.P1Hotkey2SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotkey2SpinBox, 14, 1)
+	self.P1Hotkey2Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P1Hotkey2Button, 14, 2)
+	# Hot-key3 P1
+	self.P1Hotkey3SpinBox = QDoubleSpinBox()
+	self.P1Hotkey3SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotkey3SpinBox.setValue(float(P1hotkey3))
+	self.P1Hotkey3SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotkey3SpinBox, 15, 1)
+	self.P1Hotkey3Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P1Hotkey3Button, 15, 2)
 	# Hot-key0 P2
 	self.P2Hotkey0SpinBox = QDoubleSpinBox()
 	self.P2Hotkey0SpinBox.setRange(-1, maxKeyValue)
@@ -942,38 +1001,54 @@ class keyWindow(QDialog):
 	grid.addWidget(self.P2Hotkey1SpinBox, 13, 3)
 	self.P2Hotkey1Button = MyKeyPressed("key",self)
 	grid.addWidget(self.P2Hotkey1Button, 13, 4)
+	# Hot-key2 P2
+	self.P2Hotkey2SpinBox = QDoubleSpinBox()
+	self.P2Hotkey2SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotkey2SpinBox.setValue(float(P2hotkey2))
+	self.P2Hotkey2SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotkey2SpinBox, 14, 3)
+	self.P2Hotkey2Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P2Hotkey2Button, 14, 4)
+	# Hot-key3 P2
+	self.P2Hotkey3SpinBox = QDoubleSpinBox()
+	self.P2Hotkey3SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotkey3SpinBox.setValue(float(P2hotkey3))
+	self.P2Hotkey3SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotkey3SpinBox, 15, 3)
+	self.P2Hotkey3Button = MyKeyPressed("key",self)
+	grid.addWidget(self.P2Hotkey3Button, 15, 4)
 	# Reset-key
 	self.P1ResetkeySpinBox = QDoubleSpinBox()
 	self.P1ResetkeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P1ResetkeySpinBox.setValue(float(self.P1Resetkey))
 	self.P1ResetkeySpinBox.setDecimals(0)
-	grid.addWidget(self.P1ResetkeySpinBox, 14, 1)
+	grid.addWidget(self.P1ResetkeySpinBox, 16, 1)
 	self.P1ResetButton = MyKeyPressed("key",self)
-	grid.addWidget(self.P1ResetButton, 14, 2)
+	grid.addWidget(self.P1ResetButton, 16, 2)
 	# Exit-key
 	self.P1ExitkeySpinBox = QDoubleSpinBox()
 	self.P1ExitkeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P1ExitkeySpinBox.setValue(float(self.P1Exitkey))
 	self.P1ExitkeySpinBox.setDecimals(0)
-	grid.addWidget(self.P1ExitkeySpinBox, 15, 1)
+	grid.addWidget(self.P1ExitkeySpinBox, 17, 1)
 	self.P1ExitButton = MyKeyPressed("key",self)
-	grid.addWidget(self.P1ExitButton, 15, 2)
+	grid.addWidget(self.P1ExitButton, 17, 2)
 	# Savestate-key
 	self.P1SavestatekeySpinBox = QDoubleSpinBox()
 	self.P1SavestatekeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P1SavestatekeySpinBox.setValue(float(self.P1Savestatekey))
 	self.P1SavestatekeySpinBox.setDecimals(0)
-	grid.addWidget(self.P1SavestatekeySpinBox, 16, 1)
+	grid.addWidget(self.P1SavestatekeySpinBox, 18, 1)
 	self.P1SavestateButton = MyKeyPressed("key",self)
-	grid.addWidget(self.P1SavestateButton, 16, 2)
+	grid.addWidget(self.P1SavestateButton, 18, 2)
 	# Loadstate-key
 	self.P1LoadstatekeySpinBox = QDoubleSpinBox()
 	self.P1LoadstatekeySpinBox.setRange(minKeyValue, maxKeyValue)
 	self.P1LoadstatekeySpinBox.setValue(float(self.P1Loadstatekey))
 	self.P1LoadstatekeySpinBox.setDecimals(0)
-	grid.addWidget(self.P1LoadstatekeySpinBox, 17, 1)
+	grid.addWidget(self.P1LoadstatekeySpinBox, 19, 1)
 	self.P1LoadstateButton = MyKeyPressed("key",self)
-	grid.addWidget(self.P1LoadstateButton, 17, 2)
+	grid.addWidget(self.P1LoadstateButton, 19, 2)
 
 	self.setLayout(grid)
         os.system('run-standalone.sh dbus-send --type=method_call --dest=org.freedesktop.Notifications  \
@@ -994,6 +1069,8 @@ class keyWindow(QDialog):
 	self.connect(self.P1ExitkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1Hotkey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1Hotkey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1Hotkey2SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1Hotkey3SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1SavestatekeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P1LoadstatekeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
         self.connect(self.P1RightButton, SIGNAL("keyPressed"),self.P1updateRight)
@@ -1009,6 +1086,8 @@ class keyWindow(QDialog):
         self.connect(self.P1DButton, SIGNAL("keyPressed"),self.P1updateD)
         self.connect(self.P1Hotkey0Button, SIGNAL("keyPressed"),self.P1updateHotkey0)
         self.connect(self.P1Hotkey1Button, SIGNAL("keyPressed"),self.P1updateHotkey1)
+        self.connect(self.P1Hotkey2Button, SIGNAL("keyPressed"),self.P1updateHotkey2)
+        self.connect(self.P1Hotkey3Button, SIGNAL("keyPressed"),self.P1updateHotkey3)
         self.connect(self.P1ResetButton, SIGNAL("keyPressed"),self.P1updateReset)
         self.connect(self.P1ExitButton, SIGNAL("keyPressed"),self.P1updateExit)
         self.connect(self.P1SavestateButton, SIGNAL("keyPressed"),self.P1updateSavestate)
@@ -1026,6 +1105,8 @@ class keyWindow(QDialog):
 	self.connect(self.P2RightkeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P2Hotkey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
 	self.connect(self.P2Hotkey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P2Hotkey2SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P2Hotkey3SpinBox, SIGNAL('valueChanged(double)'), self.change_value)
         self.connect(self.P2RightButton, SIGNAL("keyPressed"),self.P2updateRight)
         self.connect(self.P2LeftButton, SIGNAL("keyPressed"),self.P2updateLeft)
         self.connect(self.P2DownButton, SIGNAL("keyPressed"),self.P2updateDown)
@@ -1038,6 +1119,8 @@ class keyWindow(QDialog):
         self.connect(self.P2DButton, SIGNAL("keyPressed"),self.P2updateD)
         self.connect(self.P2Hotkey0Button, SIGNAL("keyPressed"),self.P2updateHotkey0)
         self.connect(self.P2Hotkey1Button, SIGNAL("keyPressed"),self.P2updateHotkey1)
+        self.connect(self.P2Hotkey2Button, SIGNAL("keyPressed"),self.P2updateHotkey2)
+        self.connect(self.P2Hotkey3Button, SIGNAL("keyPressed"),self.P2updateHotkey3)
 
 	# P1
     def P1updateRight(self):
@@ -1072,6 +1155,10 @@ class keyWindow(QDialog):
 	self.P1Hotkey0SpinBox.setValue(float(AsciiVal))
     def P1updateHotkey1(self):
 	self.P1Hotkey1SpinBox.setValue(float(AsciiVal))
+    def P1updateHotkey2(self):
+	self.P1Hotkey2SpinBox.setValue(float(AsciiVal))
+    def P1updateHotkey3(self):
+	self.P1Hotkey3SpinBox.setValue(float(AsciiVal))
 	# P2
     def P2updateRight(self):
 	self.P2RightkeySpinBox.setValue(float(AsciiVal))
@@ -1097,6 +1184,10 @@ class keyWindow(QDialog):
 	self.P2Hotkey0SpinBox.setValue(float(AsciiVal))
     def P2updateHotkey1(self):
 	self.P2Hotkey1SpinBox.setValue(float(AsciiVal))
+    def P2updateHotkey2(self):
+	self.P2Hotkey2SpinBox.setValue(float(AsciiVal))
+    def P2updateHotkey3(self):
+	self.P2Hotkey3SpinBox.setValue(float(AsciiVal))
 
     def change_value(self,event):
 	# P1
@@ -1112,6 +1203,8 @@ class keyWindow(QDialog):
 	self.P1Keys[9] = "%.0f" % self.P1RightkeySpinBox.value()
 	self.P1Keys[10] = "%.0f" % self.P1Hotkey0SpinBox.value()
 	self.P1Keys[11] = "%.0f" % self.P1Hotkey1SpinBox.value()
+	self.P1Keys[12] = "%.0f" % self.P1Hotkey2SpinBox.value()
+	self.P1Keys[13] = "%.0f" % self.P1Hotkey3SpinBox.value()
 	self.P1Resetkey = "%.0f" % self.P1ResetkeySpinBox.value()
 	self.P1Exitkey = "%.0f" % self.P1ExitkeySpinBox.value()
 	self.P1Savestatekey = "%.0f" % self.P1SavestatekeySpinBox.value()
@@ -1129,6 +1222,8 @@ class keyWindow(QDialog):
 	self.P2Keys[9] = "%.0f" % self.P2RightkeySpinBox.value()
 	self.P2Keys[10] = "%.0f" % self.P2Hotkey0SpinBox.value()
 	self.P2Keys[11] = "%.0f" % self.P2Hotkey1SpinBox.value()
+	self.P2Keys[12] = "%.0f" % self.P2Hotkey2SpinBox.value()
+	self.P2Keys[13] = "%.0f" % self.P2Hotkey3SpinBox.value()
 	global P1buttons
 	global P2buttons
 	global quitbutton
@@ -1179,6 +1274,12 @@ class JoykeyWindow(QDialog):
 	if len(self.P1joyKeys) < 12:
 		self.P1joyKeys.append(-1)
 	P1hotjoykey1=self.P1joyKeys[11]
+	if len(self.P1joyKeys) < 13:
+		self.P1joyKeys.append(-1)
+	P1hotjoykey2=self.P1joyKeys[12]
+	if len(self.P1joyKeys) < 14:
+		self.P1joyKeys.append(-1)
+	P1hotjoykey3=self.P1joyKeys[13]
 	# P2 keys
 	P2Ajoykey=self.P2joyKeys[0]
 	P2Bjoykey=self.P2joyKeys[1]
@@ -1197,13 +1298,19 @@ class JoykeyWindow(QDialog):
 	if len(self.P2joyKeys) < 12:
 		self.P2joyKeys.append(-1)
 	P2hotjoykey1=self.P2joyKeys[11]
+	if len(self.P2joyKeys) < 13:
+		self.P2joyKeys.append(-1)
+	P2hotjoykey2=self.P2joyKeys[12]
+	if len(self.P2joyKeys) < 14:
+		self.P2joyKeys.append(-1)
+	P2hotjoykey3=self.P2joyKeys[13]
 	# Layout
 	area = QScrollArea(self)
 	lay = QVBoxLayout(self)
 	lay.addWidget(area)
 	wg = QWidget()
 	area.setWidget(wg)
-	wg.resize(750,900);
+	wg.resize(750,1100);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -1219,7 +1326,7 @@ class JoykeyWindow(QDialog):
 		grid.addWidget(QLabel("<center>[pick]</center>"), 1, 4)
 	# Add button list
         cntr=2
-	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1']:
+	for pButton in ['A','B','C','D','START','COIN','UP','DOWN','LEFT','RIGHT','HOTKEY0','HOTKEY1','HOTKEY2','HOTKEY3']:
 		grid.addWidget(QLabel(pButton), cntr, 0)
 		cntr=cntr+1
 	# A-button P1
@@ -1366,6 +1473,30 @@ class JoykeyWindow(QDialog):
 	self.P2Hotjoykey1SpinBox.setValue(float(P2hotjoykey1))
 	self.P2Hotjoykey1SpinBox.setDecimals(0)
 	grid.addWidget(self.P2Hotjoykey1SpinBox, 13, 3)
+	# hotkey2 P1
+	self.P1Hotjoykey2SpinBox = QDoubleSpinBox()
+	self.P1Hotjoykey2SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotjoykey2SpinBox.setValue(float(P2hotjoykey2))
+	self.P1Hotjoykey2SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotjoykey2SpinBox, 14, 1)
+	# hotkey2 P2
+	self.P2Hotjoykey2SpinBox = QDoubleSpinBox()
+	self.P2Hotjoykey2SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotjoykey2SpinBox.setValue(float(P2hotjoykey2))
+	self.P2Hotjoykey2SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotjoykey2SpinBox, 14, 3)
+	# hotkey3 P1
+	self.P1Hotjoykey3SpinBox = QDoubleSpinBox()
+	self.P1Hotjoykey3SpinBox.setRange(-1, maxKeyValue)
+	self.P1Hotjoykey3SpinBox.setValue(float(P1hotjoykey3))
+	self.P1Hotjoykey3SpinBox.setDecimals(0)
+	grid.addWidget(self.P1Hotjoykey3SpinBox, 15, 1)
+	# hotkey3 P2
+	self.P2Hotjoykey3SpinBox = QDoubleSpinBox()
+	self.P2Hotjoykey3SpinBox.setRange(-1, maxKeyValue)
+	self.P2Hotjoykey3SpinBox.setValue(float(P2hotjoykey3))
+	self.P2Hotjoykey3SpinBox.setDecimals(0)
+	grid.addWidget(self.P2Hotjoykey3SpinBox, 15, 3)
 
 	if (joystickAttached):
 		self.P2AjoyButton = QPushButton("button",self)
@@ -1390,8 +1521,12 @@ class JoykeyWindow(QDialog):
 		self.P1DownjoyButton = QPushButton("button",self)
 		self.P1Hotkey0joyButton = QPushButton("button",self)
 		self.P1Hotkey1joyButton = QPushButton("button",self)
+		self.P1Hotkey2joyButton = QPushButton("button",self)
+		self.P1Hotkey3joyButton = QPushButton("button",self)
 		self.P2Hotkey0joyButton = QPushButton("button",self)
 		self.P2Hotkey1joyButton = QPushButton("button",self)
+		self.P2Hotkey2joyButton = QPushButton("button",self)
+		self.P2Hotkey3joyButton = QPushButton("button",self)
 		grid.addWidget(self.P1AjoyButton, 2, 2)
 		grid.addWidget(self.P2AjoyButton, 2, 4)
 		grid.addWidget(self.P2RightjoyButton, 11, 4)
@@ -1400,6 +1535,10 @@ class JoykeyWindow(QDialog):
 		grid.addWidget(self.P2Hotkey0joyButton, 12, 4)
 		grid.addWidget(self.P1Hotkey1joyButton, 13, 2)
 		grid.addWidget(self.P2Hotkey1joyButton, 13, 4)
+		grid.addWidget(self.P1Hotkey2joyButton, 14, 2)
+		grid.addWidget(self.P2Hotkey2joyButton, 14, 4)
+		grid.addWidget(self.P1Hotkey3joyButton, 15, 2)
+		grid.addWidget(self.P2Hotkey3joyButton, 15, 4)
 		grid.addWidget(self.P2LeftjoyButton, 10, 4)
 		grid.addWidget(self.P2DownjoyButton, 9, 4)
 		grid.addWidget(self.P1LeftjoyButton, 10, 2)
@@ -1434,6 +1573,8 @@ class JoykeyWindow(QDialog):
 	self.connect(self.P1RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P1Hotjoykey0SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P1Hotjoykey1SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1Hotjoykey2SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1Hotjoykey3SpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	if (joystickAttached):
 		self.connect(self.P1RightjoyButton, SIGNAL("clicked()"),self.P1joyupdateRight)
 		self.connect(self.P1LeftjoyButton, SIGNAL("clicked()"),self.P1updateLeft)
@@ -1447,6 +1588,8 @@ class JoykeyWindow(QDialog):
 		self.connect(self.P1DjoyButton, SIGNAL("clicked()"),self.P1updateD)
 		self.connect(self.P1Hotkey0joyButton, SIGNAL("clicked()"),self.P1updateHotkey0)
 		self.connect(self.P1Hotkey1joyButton, SIGNAL("clicked()"),self.P1updateHotkey1)
+		self.connect(self.P1Hotkey2joyButton, SIGNAL("clicked()"),self.P1updateHotkey2)
+		self.connect(self.P1Hotkey3joyButton, SIGNAL("clicked()"),self.P1updateHotkey3)
 	# P2
 	self.connect(self.P2AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	self.connect(self.P2BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
@@ -1471,6 +1614,8 @@ class JoykeyWindow(QDialog):
 		self.connect(self.P2DjoyButton, SIGNAL("clicked()"),self.P2updateD)
 		self.connect(self.P2Hotkey0joyButton, SIGNAL("clicked()"),self.P2updateHotkey0)
 		self.connect(self.P2Hotkey1joyButton, SIGNAL("clicked()"),self.P2updateHotkey1)
+		self.connect(self.P2Hotkey2joyButton, SIGNAL("clicked()"),self.P2updateHotkey2)
+		self.connect(self.P2Hotkey3joyButton, SIGNAL("clicked()"),self.P2updateHotkey3)
 
 	# P1
     def P1joyupdateRight(self):
@@ -1509,6 +1654,12 @@ class JoykeyWindow(QDialog):
     def P1updateHotkey1(self):
 	MyJoykeyPressed()
 	self.P1Hotjoykey1SpinBox.setValue(float(joyVal))
+    def P1updateHotkey2(self):
+	MyJoykeyPressed()
+	self.P1Hotjoykey2SpinBox.setValue(float(joyVal))
+    def P1updateHotkey3(self):
+	MyJoykeyPressed()
+	self.P1Hotjoykey3SpinBox.setValue(float(joyVal))
 
 	# P2
     def P2updateRight(self):
@@ -1547,6 +1698,12 @@ class JoykeyWindow(QDialog):
     def P2updateHotkey1(self):
 	MyJoykeyPressed()
 	self.P2Hotjoykey1SpinBox.setValue(float(joyVal))
+    def P2updateHotkey2(self):
+	MyJoykeyPressed()
+	self.P2Hotjoykey2SpinBox.setValue(float(joyVal))
+    def P2updateHotkey3(self):
+	MyJoykeyPressed()
+	self.P2Hotjoykey3SpinBox.setValue(float(joyVal))
 
     def change_value_joy(self):
 	# P1
@@ -1562,6 +1719,8 @@ class JoykeyWindow(QDialog):
 	self.P1joyKeys[9] = "%.0f" % self.P1RightjoykeySpinBox.value()
 	self.P1joyKeys[10] = "%.0f" % self.P1Hotjoykey0SpinBox.value()
 	self.P1joyKeys[11] = "%.0f" % self.P1Hotjoykey1SpinBox.value()
+	self.P1joyKeys[12] = "%.0f" % self.P1Hotjoykey2SpinBox.value()
+	self.P1joyKeys[13] = "%.0f" % self.P1Hotjoykey3SpinBox.value()
 	# P2
 	self.P2joyKeys[0] = "%.0f" % self.P2AjoykeySpinBox.value()
 	self.P2joyKeys[1] = "%.0f" % self.P2BjoykeySpinBox.value()
@@ -1575,6 +1734,8 @@ class JoykeyWindow(QDialog):
 	self.P2joyKeys[9] = "%.0f" % self.P2RightjoykeySpinBox.value()
 	self.P2joyKeys[10] = "%.0f" % self.P2Hotjoykey0SpinBox.value()
 	self.P2joyKeys[11] = "%.0f" % self.P2Hotjoykey1SpinBox.value()
+	self.P2joyKeys[12] = "%.0f" % self.P2Hotjoykey2SpinBox.value()
+	self.P2joyKeys[13] = "%.0f" % self.P2Hotjoykey3SpinBox.value()
 	global P1joybuttons
 	global P2joybuttons
 	P1joybuttons = self.P1joyKeys
@@ -1677,7 +1838,7 @@ class hotkeyWindow(QDialog):
 	lay.addWidget(area)
 	wg = QWidget()
 	area.setWidget(wg)
-	wg.setMinimumSize(700,300);
+	wg.setMinimumSize(700,700);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -1693,6 +1854,14 @@ class hotkeyWindow(QDialog):
 		self.P1hotKeys1=self.options["p1hotkey1"].split(",")
 	else:
 		self.P1hotKeys1 = P1hotbuttons1
+	if P1hotbuttons2 == "":
+		self.P1hotKeys2=self.options["p1hotkey2"].split(",")
+	else:
+		self.P1hotKeys2 = P1hotbuttons2
+	if P1hotbuttons3 == "":
+		self.P1hotKeys3=self.options["p1hotkey3"].split(",")
+	else:
+		self.P1hotKeys3 = P1hotbuttons3
 	if P2hotbuttons0 == "":
 		self.P2hotKeys0=self.options["p2hotkey0"].split(",")
 	else:
@@ -1701,10 +1870,22 @@ class hotkeyWindow(QDialog):
 		self.P2hotKeys1=self.options["p2hotkey1"].split(",")
 	else:
 		self.P2hotKeys1 = P2hotbuttons1
+	if P2hotbuttons2 == "":
+		self.P2hotKeys2=self.options["p2hotkey2"].split(",")
+	else:
+		self.P2hotKeys2 = P2hotbuttons2
+	if P2hotbuttons3 == "":
+		self.P2hotKeys3=self.options["p2hotkey3"].split(",")
+	else:
+		self.P2hotKeys3 = P2hotbuttons3
 	grid.addWidget(QLabel("P1 hotkey 0"), 1, 1)
 	grid.addWidget(QLabel("P1 hotkey 1"), 2, 1)
-	grid.addWidget(QLabel("P2 hotkey 0"), 4, 1)
-	grid.addWidget(QLabel("P2 hotkey 1"), 5, 1)
+	grid.addWidget(QLabel("P1 hotkey 2"), 3, 1)
+	grid.addWidget(QLabel("P1 hotkey 3"), 4, 1)
+	grid.addWidget(QLabel("P2 hotkey 0"), 5, 1)
+	grid.addWidget(QLabel("P2 hotkey 1"), 6, 1)
+	grid.addWidget(QLabel("P2 hotkey 2"), 7, 1)
+	grid.addWidget(QLabel("P2 hotkey 3"), 8, 1)
 	# p1hotkey0
         self.P1A0PushButton = QPushButton("A",self)
 	self.P1A0PushButton.setCheckable(True)
@@ -1731,32 +1912,84 @@ class hotkeyWindow(QDialog):
         self.P1D1PushButton = QPushButton("D",self)
 	self.P1D1PushButton.setCheckable(True)
         grid.addWidget(self.P1D1PushButton,2,5)
+	# p1hotkey2
+        self.P1A2PushButton = QPushButton("A",self)
+	self.P1A2PushButton.setCheckable(True)
+        grid.addWidget(self.P1A2PushButton,3,2)
+        self.P1B2PushButton = QPushButton("B",self)
+	self.P1B2PushButton.setCheckable(True)
+        grid.addWidget(self.P1B2PushButton,3,3)
+        self.P1C2PushButton = QPushButton("C",self)
+	self.P1C2PushButton.setCheckable(True)
+        grid.addWidget(self.P1C2PushButton,3,4)
+        self.P1D2PushButton = QPushButton("D",self)
+	self.P1D2PushButton.setCheckable(True)
+        grid.addWidget(self.P1D2PushButton,3,5)
+	# p1hotkey3
+        self.P1A3PushButton = QPushButton("A",self)
+	self.P1A3PushButton.setCheckable(True)
+        grid.addWidget(self.P1A3PushButton,4,2)
+        self.P1B3PushButton = QPushButton("B",self)
+	self.P1B3PushButton.setCheckable(True)
+        grid.addWidget(self.P1B3PushButton,4,3)
+        self.P1C3PushButton = QPushButton("C",self)
+	self.P1C3PushButton.setCheckable(True)
+        grid.addWidget(self.P1C3PushButton,4,4)
+        self.P1D3PushButton = QPushButton("D",self)
+	self.P1D3PushButton.setCheckable(True)
+        grid.addWidget(self.P1D3PushButton,4,5)
 	# p2hotkey0
         self.P2A0PushButton = QPushButton("A",self)
 	self.P2A0PushButton.setCheckable(True)
-        grid.addWidget(self.P2A0PushButton,4,2)
+        grid.addWidget(self.P2A0PushButton,5,2)
         self.P2B0PushButton = QPushButton("B",self)
 	self.P2B0PushButton.setCheckable(True)
-        grid.addWidget(self.P2B0PushButton,4,3)
+        grid.addWidget(self.P2B0PushButton,5,3)
         self.P2C0PushButton = QPushButton("C",self)
 	self.P2C0PushButton.setCheckable(True)
-        grid.addWidget(self.P2C0PushButton,4,4)
+        grid.addWidget(self.P2C0PushButton,5,4)
         self.P2D0PushButton = QPushButton("D",self)
 	self.P2D0PushButton.setCheckable(True)
-        grid.addWidget(self.P2D0PushButton,4,5)
+        grid.addWidget(self.P2D0PushButton,5,5)
 	# p2hotkey1
         self.P2A1PushButton = QPushButton("A",self)
 	self.P2A1PushButton.setCheckable(True)
-        grid.addWidget(self.P2A1PushButton,5,2)
+        grid.addWidget(self.P2A1PushButton,6,2)
         self.P2B1PushButton = QPushButton("B",self)
 	self.P2B1PushButton.setCheckable(True)
-        grid.addWidget(self.P2B1PushButton,5,3)
+        grid.addWidget(self.P2B1PushButton,6,3)
         self.P2C1PushButton = QPushButton("C",self)
 	self.P2C1PushButton.setCheckable(True)
-        grid.addWidget(self.P2C1PushButton,5,4)
+        grid.addWidget(self.P2C1PushButton,6,4)
         self.P2D1PushButton = QPushButton("D",self)
 	self.P2D1PushButton.setCheckable(True)
-        grid.addWidget(self.P2D1PushButton,5,5)
+        grid.addWidget(self.P2D1PushButton,6,5)
+	# p2hotkey2
+        self.P2A2PushButton = QPushButton("A",self)
+	self.P2A2PushButton.setCheckable(True)
+        grid.addWidget(self.P2A2PushButton,7,2)
+        self.P2B2PushButton = QPushButton("B",self)
+	self.P2B2PushButton.setCheckable(True)
+        grid.addWidget(self.P2B2PushButton,7,3)
+        self.P2C2PushButton = QPushButton("C",self)
+	self.P2C2PushButton.setCheckable(True)
+        grid.addWidget(self.P2C2PushButton,7,4)
+        self.P2D2PushButton = QPushButton("D",self)
+	self.P2D2PushButton.setCheckable(True)
+        grid.addWidget(self.P2D2PushButton,7,5)
+	# p2hotkey3
+        self.P2A3PushButton = QPushButton("A",self)
+	self.P2A3PushButton.setCheckable(True)
+        grid.addWidget(self.P2A3PushButton,8,2)
+        self.P2B3PushButton = QPushButton("B",self)
+	self.P2B3PushButton.setCheckable(True)
+        grid.addWidget(self.P2B3PushButton,8,3)
+        self.P2C3PushButton = QPushButton("C",self)
+	self.P2C3PushButton.setCheckable(True)
+        grid.addWidget(self.P2C3PushButton,8,4)
+        self.P2D3PushButton = QPushButton("D",self)
+	self.P2D3PushButton.setCheckable(True)
+        grid.addWidget(self.P2D3PushButton,8,5)
 
 	# A = 1, B = 2, C = 4, D = 8
 	# p1hotkey0
@@ -1793,6 +2026,40 @@ class hotkeyWindow(QDialog):
 		self.P1D1PushButton.setChecked(True)
 	else:
 		self.P1hotKeys1.append("")
+	# p1hotkey2
+	if "1" in self.P1hotKeys2:
+		self.P1A2PushButton.setChecked(True)
+	else:
+		self.P1hotKeys2.insert(0,"")
+	if "2" in self.P1hotKeys2:
+		self.P1B2PushButton.setChecked(True)
+	else:
+		self.P1hotKeys2.insert(1,"")
+	if "4" in self.P1hotKeys2:
+		self.P1C2PushButton.setChecked(True)
+	else:
+		self.P1hotKeys2.insert(2,"")
+	if "8" in self.P1hotKeys2:
+		self.P1D2PushButton.setChecked(True)
+	else:
+		self.P1hotKeys2.append("")
+	# p1hotkey3
+	if "1" in self.P1hotKeys3:
+		self.P1A3PushButton.setChecked(True)
+	else:
+		self.P1hotKeys3.insert(0,"")
+	if "2" in self.P1hotKeys3:
+		self.P1B3PushButton.setChecked(True)
+	else:
+		self.P1hotKeys3.insert(1,"")
+	if "4" in self.P1hotKeys3:
+		self.P1C3PushButton.setChecked(True)
+	else:
+		self.P1hotKeys3.insert(2,"")
+	if "8" in self.P1hotKeys3:
+		self.P1D3PushButton.setChecked(True)
+	else:
+		self.P1hotKeys3.append("")
 	# p2hotkey0
 	if "1" in self.P2hotKeys0:
 		self.P2A0PushButton.setChecked(True)
@@ -1827,6 +2094,40 @@ class hotkeyWindow(QDialog):
 		self.P2D1PushButton.setChecked(True)
 	else:
 		self.P2hotKeys1.append("")
+	# p1hotkey2
+	if "1" in self.P2hotKeys2:
+		self.P2A2PushButton.setChecked(True)
+	else:
+		self.P2hotKeys2.insert(0,"")
+	if "2" in self.P2hotKeys2:
+		self.P2B2PushButton.setChecked(True)
+	else:
+		self.P2hotKeys2.insert(1,"")
+	if "4" in self.P2hotKeys2:
+		self.P2C2PushButton.setChecked(True)
+	else:
+		self.P2hotKeys2.insert(2,"")
+	if "8" in self.P2hotKeys2:
+		self.P2D2PushButton.setChecked(True)
+	else:
+		self.P2hotKeys2.append("")
+	# p1hotkey3
+	if "1" in self.P2hotKeys3:
+		self.P2A3PushButton.setChecked(True)
+	else:
+		self.P2hotKeys3.insert(0,"")
+	if "2" in self.P2hotKeys3:
+		self.P2B3PushButton.setChecked(True)
+	else:
+		self.P2hotKeys3.insert(1,"")
+	if "4" in self.P2hotKeys3:
+		self.P2C3PushButton.setChecked(True)
+	else:
+		self.P2hotKeys3.insert(2,"")
+	if "8" in self.P2hotKeys3:
+		self.P2D3PushButton.setChecked(True)
+	else:
+		self.P2hotKeys3.append("")
 
 	# Connect to P1 hotkey buttons
 	self.connect(self.P1A0PushButton, SIGNAL("clicked()"),self.P1A0update)
@@ -1837,7 +2138,15 @@ class hotkeyWindow(QDialog):
 	self.connect(self.P1B1PushButton, SIGNAL("clicked()"),self.P1B1update)
 	self.connect(self.P1C1PushButton, SIGNAL("clicked()"),self.P1C1update)
 	self.connect(self.P1D1PushButton, SIGNAL("clicked()"),self.P1D1update)
-	# Connect to P1 hotkey buttons
+	self.connect(self.P1A2PushButton, SIGNAL("clicked()"),self.P1A2update)
+	self.connect(self.P1B2PushButton, SIGNAL("clicked()"),self.P1B2update)
+	self.connect(self.P1C2PushButton, SIGNAL("clicked()"),self.P1C2update)
+	self.connect(self.P1D2PushButton, SIGNAL("clicked()"),self.P1D2update)
+	self.connect(self.P1A3PushButton, SIGNAL("clicked()"),self.P1A3update)
+	self.connect(self.P1B3PushButton, SIGNAL("clicked()"),self.P1B3update)
+	self.connect(self.P1C3PushButton, SIGNAL("clicked()"),self.P1C3update)
+	self.connect(self.P1D3PushButton, SIGNAL("clicked()"),self.P1D3update)
+	# Connect to P2 hotkey buttons
 	self.connect(self.P2A0PushButton, SIGNAL("clicked()"),self.P2A0update)
 	self.connect(self.P2B0PushButton, SIGNAL("clicked()"),self.P2B0update)
 	self.connect(self.P2C0PushButton, SIGNAL("clicked()"),self.P2C0update)
@@ -1846,6 +2155,14 @@ class hotkeyWindow(QDialog):
 	self.connect(self.P2B1PushButton, SIGNAL("clicked()"),self.P2B1update)
 	self.connect(self.P2C1PushButton, SIGNAL("clicked()"),self.P2C1update)
 	self.connect(self.P2D1PushButton, SIGNAL("clicked()"),self.P2D1update)
+	self.connect(self.P2A2PushButton, SIGNAL("clicked()"),self.P2A2update)
+	self.connect(self.P2B2PushButton, SIGNAL("clicked()"),self.P2B2update)
+	self.connect(self.P2C2PushButton, SIGNAL("clicked()"),self.P2C2update)
+	self.connect(self.P2D2PushButton, SIGNAL("clicked()"),self.P2D2update)
+	self.connect(self.P2A3PushButton, SIGNAL("clicked()"),self.P2A3update)
+	self.connect(self.P2B3PushButton, SIGNAL("clicked()"),self.P2B3update)
+	self.connect(self.P2C3PushButton, SIGNAL("clicked()"),self.P2C3update)
+	self.connect(self.P2D3PushButton, SIGNAL("clicked()"),self.P2D3update)
 
 	# player 1 hotkey update
     def P1A0update(self):
@@ -1911,7 +2228,71 @@ class hotkeyWindow(QDialog):
 	else:
 		self.P1hotKeys1[3] = ""
 	P1hotbuttons1 = self.P1hotKeys1
+
+    def P1A2update(self):
+	global P1hotbuttons2
+	if self.P1A2PushButton.isChecked():
+		self.P1hotKeys2[0] = "1"
+	else:
+		self.P1hotKeys2[0] = ""
+	P1hotbuttons2 = self.P1hotKeys2
+
+    def P1B2update(self):
+	global P1hotbuttons2
+	if self.P1B2PushButton.isChecked():
+		self.P1hotKeys2[1] = "2"
+	else:
+		self.P1hotKeys2[1] = ""
+	P1hotbuttons2 = self.P1hotKeys2
  
+    def P1C2update(self):
+	global P1hotbuttons2
+	if self.P1C2PushButton.isChecked():
+		self.P1hotKeys2[2] = "4"
+	else:
+		self.P1hotKeys2[2] = ""
+	P1hotbuttons2 = self.P1hotKeys2
+ 
+    def P1D2update(self):
+	global P1hotbuttons2
+	if self.P1D2PushButton.isChecked():
+		self.P1hotKeys2[3] = "8"
+	else:
+		self.P1hotKeys2[3] = ""
+	P1hotbuttons2 = self.P1hotKeys2
+ 
+    def P1A3update(self):
+	global P1hotbuttons3
+	if self.P1A3PushButton.isChecked():
+		self.P1hotKeys3[0] = "1"
+	else:
+		self.P1hotKeys3[0] = ""
+	P1hotbuttons3 = self.P1hotKeys3
+
+    def P1B3update(self):
+	global P1hotbuttons3
+	if self.P1B3PushButton.isChecked():
+		self.P1hotKeys3[1] = "2"
+	else:
+		self.P1hotKeys3[1] = ""
+	P1hotbuttons3 = self.P1hotKeys3
+ 
+    def P1C3update(self):
+	global P1hotbuttons3
+	if self.P1C3PushButton.isChecked():
+		self.P1hotKeys3[2] = "4"
+	else:
+		self.P1hotKeys3[2] = ""
+	P1hotbuttons3 = self.P1hotKeys3
+ 
+    def P1D3update(self):
+	global P1hotbuttons3
+	if self.P1D3PushButton.isChecked():
+		self.P1hotKeys3[3] = "8"
+	else:
+		self.P1hotKeys3[3] = ""
+	P1hotbuttons3 = self.P1hotKeys3
+
 	# player 2 hotkey update
     def P2A0update(self):
 	global P2hotbuttons0
@@ -1976,6 +2357,69 @@ class hotkeyWindow(QDialog):
 	else:
 		self.P2hotKeys1[3] = ""
 	P2hotbuttons1 = self.P2hotKeys1
+    def P2A2update(self):
+	global P2hotbuttons2
+	if self.P2A2PushButton.isChecked():
+		self.P2hotKeys2[0] = "1"
+	else:
+		self.P2hotKeys2[0] = ""
+	P2hotbuttons2 = self.P2hotKeys2
+
+    def P2B2update(self):
+	global P2hotbuttons2
+	if self.P2B2PushButton.isChecked():
+		self.P2hotKeys2[1] = "2"
+	else:
+		self.P2hotKeys2[1] = ""
+	P2hotbuttons2 = self.P2hotKeys2
+ 
+    def P2C2update(self):
+	global P2hotbuttons2
+	if self.P2C2PushButton.isChecked():
+		self.P2hotKeys2[2] = "4"
+	else:
+		self.P2hotKeys2[2] = ""
+	P2hotbuttons2 = self.P2hotKeys2
+ 
+    def P2D2update(self):
+	global P2hotbuttons2
+	if self.P2D2PushButton.isChecked():
+		self.P2hotKeys2[3] = "8"
+	else:
+		self.P2hotKeys2[3] = ""
+	P2hotbuttons2 = self.P2hotKeys2
+
+    def P2A3update(self):
+	global P2hotbuttons3
+	if self.P2A3PushButton.isChecked():
+		self.P2hotKeys3[0] = "1"
+	else:
+		self.P2hotKeys3[0] = ""
+	P2hotbuttons3 = self.P2hotKeys3
+
+    def P2B3update(self):
+	global P2hotbuttons3
+	if self.P2B3PushButton.isChecked():
+		self.P2hotKeys3[1] = "2"
+	else:
+		self.P2hotKeys3[1] = ""
+	P2hotbuttons3 = self.P2hotKeys3
+ 
+    def P2C3update(self):
+	global P2hotbuttons3
+	if self.P2C3PushButton.isChecked():
+		self.P2hotKeys3[2] = "4"
+	else:
+		self.P2hotKeys3[2] = ""
+	P2hotbuttons3 = self.P2hotKeys3
+ 
+    def P2D3update(self):
+	global P2hotbuttons3
+	if self.P2D3PushButton.isChecked():
+		self.P2hotKeys3[3] = "8"
+	else:
+		self.P2hotKeys3[3] = ""
+	P2hotbuttons3 = self.P2hotKeys3
  
  
 class pathWindow(QDialog):
